/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mircoders.media;

import mir.entity.Entity;
import mir.log.LoggerWrapper;
import mir.media.MediaFailure;
import mir.media.MediaHandler;
import mir.media.MediaExc;
import mir.misc.StringUtil;
import mir.session.UploadedFile;
import mir.storage.Database;
import mir.storage.DatabaseExc;
import mir.util.FileRoutines;
import mircoders.module.ModuleMediaType;

import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;

public class MediaUploadProcessor {
  private static LoggerWrapper logger = new LoggerWrapper("Media.UploadProcessor");
  private static ModuleMediaType mediaTypeModule = new ModuleMediaType();

  /**
   * Processes an uploaded media file.
   * Will create the media entity and so on. 
   */
  public static Entity processMediaUpload(UploadedFile aFile, Map aValues) 
      throws UnsupportedMediaTypeExc, MediaExc, DatabaseExc {
    MediaHandler mediaHandler;
    Entity mediaType = null;
    Database mediaStorage;
    Map values = new HashMap();
    Entity mediaEntity;

    String contentType = aFile.getContentType();
    logger.debug("processing media upload of " + aFile.getFileName() + " (content type = " + contentType + ")");

    if (contentType!=null) {
      mediaType = mediaTypeModule.findMediaTypeForMimeType(contentType);
    }

    if (mediaType==null) {
      String extension = FileRoutines.getExtension(aFile.getFileName());
      mediaType = mediaTypeModule.findMediaTypeForExtension(extension);
    }

    if (mediaType==null) {
      throw new UnsupportedMediaTypeExc("Media " + aFile.getFileName() + " " +
          (aFile!=null?"("+aFile.getContentType()+") ":"") + " not supported", aFile.getContentType());
    }

    values.putAll(aValues);
    values.put("date", StringUtil.date2webdbDate(new GregorianCalendar()));
    values.put("is_produced", "0");

    try {
      mediaHandler = MediaHelper.getHandler(mediaType);
      mediaStorage = MediaHelper.getStorage(mediaType, mediaType.getFieldValue("tablename"));
    }
    catch (Throwable e) {
      throw new MediaFailure(e);
    }

    values.put("to_media_type", mediaType.getId());

    mediaEntity = mediaStorage.createNewEntity();
    mediaEntity.setFieldValues(values);
    mediaEntity.insert();
    mediaHandler.store(aFile, mediaEntity, mediaType);
    try {
    }
    catch (Throwable e) {
      throw new MediaFailure(e);
    }

    return mediaEntity;
  }
}