/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */

package mircoders.storage;

import mir.log.LoggerWrapper;
import mir.storage.Database;
import mir.storage.DatabaseFailure;

import java.sql.Connection;
import java.sql.Statement;

public class DatabaseContent extends Database  {

  private static DatabaseContent      instance;

  public synchronized static DatabaseContent getInstance() {

    if (instance == null ) {
      instance = new DatabaseContent();
    }
    return instance;
  }

  private DatabaseContent() throws DatabaseFailure {

    super();
    mainTable="content";
    logger = new LoggerWrapper("Database.Content");

    entityClass = mircoders.entity.EntityContent.class;
  }

  // methods

  /**
   * sets the database flag is_produced to unproduced
   */

  public void setUnproduced(String where) throws DatabaseFailure
  {
    Connection con=null;Statement stmt=null;
    String sql = "update content set is_produced='0' where " + where;
    logger.debug("set unproduced: "+where);
    try {
      con = obtainConnection();
      // should be a preparedStatement because is faster
      stmt = con.createStatement();
      executeUpdate(stmt,sql);
      logger.debug("set unproduced: "+where);
    }
    catch (Exception e) {
      _throwStorageObjectException(e, "-- set unproduced failed");
    }
    finally { freeConnection(con,stmt);}
  }

  public boolean delete(String id) throws DatabaseFailure {
    DatabaseComment.getInstance().deleteByContentId(id);
    DatabaseContentToTopics.getInstance().deleteByContentId(id);
    DatabaseContentToMedia.getInstance().deleteByContentId(id);

    return super.delete(id);
  }

  protected String getPrimaryKeySequence() {
    return "media_id_seq";
  }
}
